package cmd

import (
	"fmt"
	"os"
	"strings"

	"git.ar21.de/yolokube/grafana-backuper/internal/config"
	"git.ar21.de/yolokube/grafana-backuper/internal/version"
	log "github.com/sirupsen/logrus"
	"github.com/spf13/cobra"
	"github.com/spf13/pflag"
	"github.com/spf13/viper"
)

const envVarPrefix = "GB"

func NewRootCommand(c *config.Config) *cobra.Command {
	rootCmd := &cobra.Command{
		Use:     "grafana-backuper",
		Short:   "Grafana Backuper CLI",
		Long:    "A command-line tool to back up and restore Grafana dashboards",
		Version: version.Version(),
		PersistentPreRun: func(cmd *cobra.Command, _ []string) {
			initializeConfig(cmd)

			c.Output = os.Stdout

			log.SetOutput(c.Output)
			log.SetLevel(log.InfoLevel)
			cmd.SetOut(c.Output)

			if c.Debug {
				log.SetLevel(log.DebugLevel)
			}
		},
		SilenceUsage: true,
	}

	rootCmd.SetErrPrefix("Error:")

	rootCmd.PersistentFlags().BoolVarP(&c.Debug, "debug", "d", false, "Debug output")
	rootCmd.PersistentFlags().StringVar(&c.GrafanaURL, "grafana-url", "", "Grafana URL to access the API")
	rootCmd.PersistentFlags().StringVar(&c.GrafanaToken, "grafana-token", "", "Grafana auth token to access the API")
	rootCmd.PersistentFlags().StringVar(&c.GitBranch, "git-branch", "main", "Git branch name")
	rootCmd.PersistentFlags().StringVar(&c.GitRepo, "git-repo", "", "Complete Git repository URL")
	rootCmd.PersistentFlags().StringVar(&c.GitUser, "git-user", "", "Git user name")
	rootCmd.PersistentFlags().StringVar(&c.GitPass, "git-pass", "", "Git user password")

	return rootCmd
}

func initializeConfig(cmd *cobra.Command) {
	v := viper.New()
	v.SetEnvKeyReplacer(strings.NewReplacer("-", "_"))
	v.SetEnvPrefix(envVarPrefix)
	v.AutomaticEnv()
	bindFlags(cmd, v)
}

func bindFlags(cmd *cobra.Command, v *viper.Viper) {
	cmd.Flags().VisitAll(func(flag *pflag.Flag) {
		// Apply the viper config value to the flag when the flag is not set and viper has a value
		if !flag.Changed && v.IsSet(flag.Name) {
			err := cmd.Flags().Set(
				flag.Name,
				fmt.Sprintf("%v", v.Get(flag.Name)),
			)
			if err != nil {
				fmt.Fprintf(os.Stderr, "Error: %v\n", err)
			}
		}
	})
}
